/* 
Trivial example web client, using raw network sockets.

References:
    "Beej's Guide to Network Programming"
    http://beej.us/guide/bgnet/output/html/singlepage/bgnet.html

    "Creating a Basic Winsock Application", by Microsoft
    https://msdn.microsoft.com/en-us/library/windows/desktop/ms737629(v=vs.85).aspx

Dr. Orion Lawlor, lawlor@alaska.edu, 2016-03-24 (public domain)
*/
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <map>
#include <string>


// Socket headers:
#if defined(_WIN32) && ! defined(__CYGWIN__)
/* For windows systems:*/

//#  include <winsock.h> /* for SOCKET and others */
//#  pragma comment (lib, "wsock32.lib")  /* link with winsock library */

#  include <winsock2.h> /* new style, for getaddrinfo */
#  include <WS2tcpip.h> /* new style, for getaddrinfo */
#  pragma comment(lib, "Ws2_32.lib")
inline void close_socket_portable(int sock) { closesocket(sock); }

class initialize_winsock {
public:
	initialize_winsock() {
		WSADATA wsaData;
		WSAStartup(MAKEWORD(2,2), &wsaData);
	}
};
initialize_winsock initialize_winsock_now;

#else 
/* Non-windows (UNIX-y) systems: */
#  include <sys/types.h>
#  include <sys/socket.h>
#  include <netdb.h>
#  include <unistd.h>
inline void close_socket_portable(int sock) { close(sock); }
#endif

// Check if an error (negative return value) came back from this code.
#define SKT_CHECK(code) { int status=code; if (status<0) { perror(#code); std::cout<<"Error with code "<<#code<<" on line "<<__LINE__<<" of file "<<__FILE__<<"\n"; exit(1); } }

#include <sstream> // for ostringstream
#include <string.h> // for memset

// Read a newline-terminated string from this socket.
std::string recv_text_line(int socket) {
	std::string req="";
	char last_char=0;
	while (1) { 
		char c=0;
		SKT_CHECK(recv(socket,&c,1,0));
	
		if (c=='\n' || c=='\r') { // newline char
			if (last_char=='\r' && c=='\n') // full newline
				break;
		}
		else req+=c;
		last_char=c;
	}
	return req;
}


int main() {
	struct addrinfo hints;
	struct addrinfo *res;  // will point to the results

	memset(&hints, 0, sizeof hints); // make sure the struct is empty
	hints.ai_family = AF_UNSPEC;     // don't care IPv4 or IPv6
	hints.ai_socktype = SOCK_STREAM; // TCP stream sockets
	hints.ai_flags = AI_PASSIVE;     // fill in my IP for me
	
	std::string host="lawlor.cs.uaf.edu";
	std::string protocol="http";
	
	SKT_CHECK(getaddrinfo(host.c_str(),protocol.c_str(), &hints,&res));

	int s=socket(res->ai_family, res->ai_socktype, res->ai_protocol);
	
	SKT_CHECK(connect(s,res->ai_addr,res->ai_addrlen)); // client
	
	freeaddrinfo(res);
	

/* Send off HTTP request to server (with URL) */
	const char *req=
		"GET / HTTP/1.1\r\n"  // the "/" is the URL I'm requesting
		"Host: lawlor.cs.uaf.edu\r\n" // hostname is required for HTTP 1.1
		"User-Agent: Raw socket example code (lawlor@alaska.edu)\r\n" // web browser ID string
		"\r\n"; // blank line == end of HTTP request
	SKT_CHECK(send(s,req,strlen(req),0));

/* Receive HTTP response headers, up to the newline */
	std::string response;
	int length=0;
	while (1) // (response=skt_recv_line(s))!="")
	{
		std::string response=recv_text_line(s);
		if (response=="") break;
		
		std::cout<<response<<"\n";
		if (response.substr(0,15)=="Content-Length:")
			length=atoi(response.substr(16).c_str());
	}

/* Receive HTTP response data, and print it */
	std::cout<<"-- bottom line: "<<length<<" bytes of data\n";
	if (length>0 && length<10000) { // sanity check
		std::vector<char> page(length); // place to store data
		SKT_CHECK(recv(s,&page[0],length,0));  // grab data from server
		for (int i=0;i<length;i++) std::cout<<page[i]; // print to screen
	}

	close_socket_portable(s);
	return 0;
}





